@@ -289,7 +289,7 @@ gom_resource_set_repository (GomResource   *resource,
    }
 }
 
-static gboolean
+gboolean
 gom_resource_do_delete (GomResource  *resource,
                         GomAdapter   *adapter,
                         GError      **error)
@@ -544,8 +544,8 @@ has_primary_key (GomResource *resource)
    return ret;
 }
 
-static void
-set_post_save_properties (GomResource *resource)
+void
+gom_resource_set_post_save_properties (GomResource *resource)
 {
    GValue *value;
 
@@ -612,7 +612,7 @@ gom_resource_build_save_cmd (GomResource *resource,
    for (iter = types; iter; iter = iter->next) {
       GomCommand *command;
 
-      resource_type = GPOINTER_TO_INT(iter->data);
+      resource_type = (GType) iter->data;
 
       g_object_set(builder,
                    "resource-type", resource_type,
@@ -708,6 +708,8 @@ gom_resource_save_cb (GomAdapter *adapter,
 
    if (!(ret = gom_resource_do_save(resource, adapter, &error))) {
       g_simple_async_result_take_error(simple, error);
+   } else {
+     g_object_set_data(G_OBJECT(resource), "is-from-table", GINT_TO_POINTER(TRUE));
    }
 
    g_simple_async_result_set_op_res_gboolean(simple, ret);
@@ -764,7 +766,7 @@ gom_resource_save_sync (GomResource  *resource,
    }
 
    if (ret)
-      set_post_save_properties(resource);
+      gom_resource_set_post_save_properties(resource);
 
    g_object_unref(simple);
 
@@ -814,7 +816,7 @@ gom_resource_save_finish (GomResource   *resource,
    }
 
    if (ret)
-      set_post_save_properties(resource);
+      gom_resource_set_post_save_properties(resource);
 
    g_object_unref(simple);
 
@@ -842,8 +844,7 @@ gom_resource_fetch_m2m_cb (GomAdapter *adapter,
    g_return_if_fail(G_IS_SIMPLE_ASYNC_RESULT(simple));
 
    m2m_table = g_object_get_data(G_OBJECT(simple), "m2m-table");
-   resource_type = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(simple),
-                                                     "resource-type"));
+   resource_type = (GType) g_object_get_data(G_OBJECT(simple), "resource-type");
    filter = g_object_get_data(G_OBJECT(simple), "filter");
    resource = GOM_RESOURCE(g_async_result_get_source_object(G_ASYNC_RESULT(simple)));
    repository = gom_resource_get_repository(resource);
